"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetupOpenVpn = void 0;
const chalk_1 = __importDefault(require("chalk"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const fs_1 = __importDefault(require("fs"));
const package_json_1 = __importDefault(require("../package.json"));
const shelljs_1 = __importDefault(require("shelljs"));
const acm_utils_1 = require("./utils/acm-utils");
class SetupOpenVpn {
    constructor(options) {
        this.options = options;
        this.acmClient = new aws_sdk_1.default.ACM({
            region: this.options.region,
            apiVersion: '2015-12-08'
        });
        this.run();
    }
    run() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Running openvpn unattended setup ${package_json_1.default.version}`));
            this.setupOpenVpn();
            this.setupSsl();
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Something failed during openvpn setup`), e);
            return;
        }
    }
    setupOpenVpn() {
        const { userName, userPassword, ip, hostName } = this.options;
        if (typeof ip !== 'undefined') {
            console.log(chalk_1.default.bgGreenBright('Setting upstream dns settings and access server hostname'));
            try {
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.client.routing.reroute_dns --value custom ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.server.dhcp_option.dns.0 --value ${ip} ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.server.routing.gateway_access --value true ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key host.name --value ${hostName} ConfigPut`);
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright('An error occured while setting upstream dns settings'), e);
                throw e;
            }
        }
        try {
            console.log(chalk_1.default.bgGreenBright('Creating default client user for OpenVPN'));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.client.routing.reroute_gw --value true ConfigPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key type --value user_connect UserPropPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key prop_autologin --value true UserPropPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName}  --new_pass ${userPassword} SetLocalPassword`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while creating default client user'), e);
            throw e;
        }
    }
    setupSsl() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(chalk_1.default.bgGreenBright(`Starting process to append an SSL certificate to OpenVPN Access Server Web UI`));
            const { domainName, email } = this.options;
            this.installCertbot();
            const certificateSummary = yield acm_utils_1.getCertificate(this.acmClient, this.options.domainName);
            if (typeof certificateSummary === 'undefined') {
                console.log(chalk_1.default.yellowBright(`No existing certificate to reuse, creating new cert`));
                try {
                    shelljs_1.default.exec([
                        'sudo',
                        'certbot',
                        'certonly',
                        '--standalone',
                        '--non-interactive',
                        '--agree-tos',
                        '--email',
                        email,
                        '--domains',
                        domainName,
                        ...(this.options.certEnvironment === 'staging'
                            ? [
                                '--server',
                                'https://acme-staging-v02.api.letsencrypt.org/directory'
                            ]
                            : [])
                    ].join(' '));
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright('An error occured while generating a cert'), e);
                    throw e;
                }
                try {
                    shelljs_1.default.exec(`sudo chmod -R 755 /etc/letsencrypt`);
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright('An error while chaging directory permissions'), e);
                    throw e;
                }
            }
            else {
                console.log(chalk_1.default.bgGreenBright(`Existing cert found, reusing it...`));
                try {
                    shelljs_1.default.exec([
                        'sudo',
                        'mkdir',
                        '-p',
                        `/etc/letsencrypt/live/${domainName}`
                    ].join(' '));
                    shelljs_1.default.exec(['sudo', 'chmod', '-R', '777', `/etc/letsencrypt`].join(' '));
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured while creating relevant directories to store certificate`), e);
                    throw e;
                }
                try {
                    const certificateResponse = yield this.acmClient
                        .exportCertificate()
                        .promise();
                    if (typeof certificateResponse.Certificate !== 'undefined' &&
                        typeof certificateResponse.CertificateChain !==
                            'undefined' &&
                        typeof certificateResponse.PrivateKey !== 'undefined') {
                        this.writeToFile(certificateResponse.Certificate, `/etc/letsencrypt/live/${domainName}/cert.pem`);
                        this.writeToFile(certificateResponse.PrivateKey, `/etc/letsencrypt/live/${domainName}/privkey.pem`);
                        this.writeToFile(certificateResponse.CertificateChain, `/etc/letsencrypt/live/${domainName}/chain.pem`);
                    }
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured while getting certificate artifacts from ACM`), e);
                    throw e;
                }
            }
            this.installCertificate();
            this.startOpenVpn();
        });
    }
    writeToFile(content, path) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                fs_1.default.writeFileSync(path, content);
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright(`Failed to write ${path}`));
                throw e;
            }
        });
    }
    installCertbot() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Installing certbot`));
            shelljs_1.default.exec(`sudo apt-get -y install software-properties-common`);
            shelljs_1.default.exec(`sudo add-apt-repository ppa:certbot/certbot -y`);
            shelljs_1.default.exec(`sudo apt-get -y update`);
            shelljs_1.default.exec(`sudo apt-get -y install certbot`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while installing certbot'), e);
            throw e;
        }
    }
    installCertificate() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Delete existing cert references in OpenVPN DB`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.ca_bundle`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.priv_key`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.cert`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while deleting certificate references in OpenVPN DB'), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Apply symbolic links to SSL certificates`));
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/cert.pem /usr/local/openvpn_as/etc/web-ssl/server.crt`);
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/privkey.pem /usr/local/openvpn_as/etc/web-ssl/server.key`);
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/chain.pem /usr/local/openvpn_as/etc/web-ssl/ca.crt`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured symlinking to certificate files'), e);
            throw e;
        }
    }
    stopOpenVpn() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Stopping openvpn`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli stop`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while stopping openvpn'), e);
            return;
        }
    }
    startOpenVpn() {
        try {
            console.log(chalk_1.default.blueBright(`Starting openvpn`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli start`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while starting openvpn'), e);
            return;
        }
    }
}
exports.SetupOpenVpn = SetupOpenVpn;
//# sourceMappingURL=openvpn-unattended-install.js.map